/*
 * ProxyServlet.java
 *
 * @version 1.0
 * @author American Management Systems
 *
 * Install ProxyServlet.class and ServerConnection.class in the servlet
 * directory on your web server.  The servlet.properties file should be
 * configured as follows:
 *
 * servlet.ProxyServlet.code=ProxyServlet
 * servlet.ProxyServlet.initArgs=serverPort=4321,serverAddress=127.0.0.1
 *
 * Where serverPort and serverAddress are set to the Applications servers
 * address.  Any messages POSTED to this proxy servlet will be fowarded
 * onto this address.
 */

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;

public class ProxyServlet extends HttpServlet {

   //Process Status Constants
   private static final int DATA_PROCESSED = 0;
   private static final int DATA_TO_BE_RECEIVED = 1;
   private static final int DATA_TO_BE_SENT = 2;
   private static final int DATA_ERROR = 3;
   private static final int DATA_CONNECTION_ERROR = 4;

   // Connection parameters global to all ProxyServlets
   private String address;
   private String port;
   private ServletOutputStream out;

  /**
   * Proxy the POST data to the Application Server and then
   * Proxy the response string data back to the client
   *
   * @param      req Request object the servlet uses to get input.
   * @param      res Response object that the servlet uses to send output.
   *
   * @exception  ServletException @see HttpServlet
   * @exception  IOException occurs due to general network errors.
   */
   public void doPost(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
   {

      String encServerData;
      byte[] postedBytes = null;
      int offset;
      int inputLen;

      // Get Address, Port from parameters
      address = getInitParameter("serverAddress");
      port = getInitParameter("serverPort");

      // IE4.0 Sends data that getParameter cannot parse.  To get around
      // this read that data straight from the stream.
      try {
         int len = req.getContentLength();
         ServletInputStream in = req.getInputStream();
         try {
            postedBytes = new byte [len];
            offset = 0;
            do {
               inputLen = in.read (postedBytes, offset, len - offset);
               if (inputLen <= 0)
                  throw new IOException ("short read");
               offset += inputLen;
            } while ((len - offset) > 0);

         } catch (IOException e) {
	        log("ProxyServlet: Error communications error receiving POST.");
	        return;
	     }

         encServerData = new String (postedBytes, 0, 0, len);
         res.setContentType("text/html");
         out = res.getOutputStream();

         if (len != 0) {
            //URL decode the query string
            String serverData = urlDecode(encServerData);

            // Send Request to Application Server
            process(serverData, out);
         } else {
            returnError("No data posted");
            return;
         }

      } catch (Exception e) {
         log("ProxyServlet: Exception occurred in doPost");
         return;
      }
   }

  /**
   * Obtain information on this servlet.
   *
   * @return String describing this servlet.
   */
   public String getServletInfo() {
      return "Proxy servlet -- proxy data to Application Server";
   }

   /**
   * Proxy the POST data to the Application Server and then
   * Proxy the response string data back to the client
   *
   * @param      request Request to send to the Application Server
   * @param      sourceOut ServletOutputStream to pipe data back to client
   */
   public void process(String request, ServletOutputStream sourceOut) {

      String err;
      String response;
      int dataState;
      int retry = 0;
      ServerConnection serverConnection;

      serverConnection = new ServerConnection(address, port);
      if (serverConnection.error()) {
         dataState = DATA_ERROR;
      } else {
         dataState = DATA_TO_BE_SENT;
      }

      while ((retry < 5) &&
             (dataState != DATA_PROCESSED) &&
             (dataState != DATA_CONNECTION_ERROR))
      {
         serverConnection.send(request);
         if (serverConnection.error()) {
            dataState = DATA_ERROR;
         } else {
            dataState = DATA_TO_BE_RECEIVED;
         }

         if (dataState == DATA_TO_BE_RECEIVED) {
            response = serverConnection.receive();
            if (serverConnection.error()) {
               dataState = DATA_ERROR;
            } else {
               dataState = DATA_PROCESSED;
               try {
                  sourceOut.println(response);
                  sourceOut.flush();
               } catch (Exception se) {
                  // Send to Server Error
                  log("ProxyServlet: Servlet send error");
               }
            }
         }

         if (dataState == DATA_ERROR) {
            // Dispose of server connection and retrieve a new one
            serverConnection.close();
            serverConnection = new ServerConnection(address, port);
            if (serverConnection.error()) {
               dataState = DATA_ERROR;
            } else {
               dataState = DATA_TO_BE_SENT;
            }
            retry++;
         }
      }

      if (dataState == DATA_PROCESSED) {
         serverConnection.close();
      } else {
         returnError("Could not connect to Application Server.");
      }
   }

  /**
   * Transmit a error message to the Client
   *
   * @param     error Error string to send back to client
   */
   public void returnError(String error) {
      String message;
      String header;
      int messageSize;

      message = "Error=" + error;
      try {
         out.print(message);
         out.flush();
      } catch (Exception e) {
          log("Proxy Servlet Communication Error");
      }
   }

  /**
   * Decode a URLencoded string
   *
   * @param      in  url data to decode
   * @return     url decoded String
   */
   private String urlDecode(String in) {
      StringBuffer out = new StringBuffer(in.length());
      int i = 0;
      int j = 0;

      while (i < in.length()) {
         char ch = in.charAt(i);
         i++;
         if (ch == '+') ch = ' ';
         else if (ch == '%') {
            ch = (char)Integer.parseInt(in.substring(i,i+2), 16);
            i+=2;
         }
         out.append(ch);
         j++;
      }
      return new String(out);
   }
}


